/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* gs_translate.c							*/
/*									*/
/*  Copyright (C) 1997,1998 Angelo Masci				*/
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  angelo@styx.demon.co.uk						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id$
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>

#include "common.h"
#include "logfile.h"

#include "gs_token.h"
#include "gs_list.h"
#include "gs_translate.h"
#include "gs_parser.h"

/* -------------------------------------------------------------------- */
/* This layer of the GS Parser should hide as much of the internal	*/
/* structures as possible.						*/
/* The only types that a user should be aware are:			*/
/* 									*/
/* 	TOKEN								*/
/* 	TOKEN_LIST							*/
/*									*/
/* -------------------------------------------------------------------- */

TOKEN *gs_getlistelement(TOKEN *item, int index);
TOKEN *gs_getlistlength(TOKEN *item);
TOKEN *gs_getlistfirstitem(TOKEN *item);
TOKEN *gs_getlistnextitem(TOKEN *item, TOKEN_LIST **ptr);


TOKEN *gs_getdictionaryfirstitem(TOKEN *item);
TOKEN *gs_getdictionarynextitem(TOKEN *item, TOKEN_LIST **ptr);

int gs_savedictionarytofile(char *filename, TOKEN *dictionary);
int gs_loaddictionarytofile(char *filename, TOKEN *dictionary);

/* -------------------------------------------------------------------- */

TOKEN *gs_get_variable(TOKEN *dictionary, char *string);

int gs_isidentifier(TOKEN *item);
int gs_isdictionary(TOKEN *item);
int gs_islist(TOKEN *item);

TOKEN *create_dictionary(char *name);

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int gs_savedictionarytofile(char *filename, TOKEN *dictionary)
{
	FILE	*fp;
	TOKEN_HEAP
		*heap;


	heap = get_token_indirect_dictionary(NULL, dictionary);
	if (heap == NULL)
	{	return -1;
	}


	fp = fopen(filename, "w");
	if (fp == NULL)
	{	return -1;
	}


	dump_heaptofile(heap, "X", fp);


	fclose(fp);
	return 0;
}



int gs_loaddictionaryfromfile(char *filename, TOKEN *dictionary)
{
	TOKEN_HEAP 
		*heap;

	TOKEN	*token_id;


	if (! gs_isdictionary(dictionary))
	{	return -1;
	}


	init_builtin_heap();
	heap = get_token_indirect_dictionary(NULL, dictionary);
	if (heap == NULL)
	{	return -1;
	}

	if (gs_open(filename))
	{	return -1;
	}

	if (parse_dictionary(heap))
	{
		token_id = get_next_token(heap);
		if (token_id != eof_tok)
		{
			gs_close();
			return -1;
		}
	}
	else
	{	gs_close();
		return -1;
	}
	
	if (gs_close())
	{	return -1;
	}

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int gs_isidentifier(TOKEN *item)
{
	if (item != NULL)
	{
		if (item->ptr_type == TP_STRING)
		{	return TRUE;
		}
	}

	return FALSE;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int gs_isdictionary(TOKEN *item)
{
	if (item != NULL)
	{
		if (item->ptr_type == TP_DICTIONARY)
		{	return TRUE;
		}
	}

	return FALSE;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int gs_islist(TOKEN *item)
{
	if (item != NULL)
	{
		if (item->ptr_type == TP_LIST)
		{	return TRUE;
		}
	}

	return FALSE;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
TOKEN *gs_get_variable(TOKEN *dictionary, char *string)
{
	TOKEN_HEAP
		*heap;

	heap = get_token_indirect_dictionary(NULL, dictionary);
	if (heap == NULL)
	{	return NULL;
	}

	return get_var(heap, string);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
TOKEN *create_dictionary(char *name)
{
	TOKEN_HEAP 
		*dictionary;

	dictionary = generate_new_heap();
	if (dictionary == NULL)
	{	return NULL;
	}

	return new_token(name, T_STRING, TP_DICTIONARY, dictionary);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int main(int argc, char **argv)
{
	TOKEN *item;


	set_logfile(LOGFILE);
	set_loglevel(LOGLEVEL);
	set_consolelog(TRUE);



	item = create_dictionary("test1");

	if (gs_isdictionary(item))
	{	lprintf(LOG_STANDARD, "Item is Dictionary\n");
	}
	else
	{	lprintf(LOG_STANDARD, "Item is NOT Dictionary\n");
	}


	if (gs_loaddictionaryfromfile("test_fileX", item) == -1)
	{	lprintf(LOG_ERROR, "Failed to load Dictionary 'test_fileX'\n");
	}
	else
	{	lprintf(LOG_STANDARD, "Loaded Dictionary\n");
	}



	if (gs_savedictionarytofile("myfile", item) == -1)
	{	lprintf(LOG_ERROR, "Failed to Save Dictionary 'myfile'\n");
	}
	else
	{	lprintf(LOG_STANDARD, "Saved Dictionary\n");
	}

	return 0;
}
